@@ -55,7 +55,7 @@ class YAML(object):
                             "one was given ({!r})".format(self.__class__.__name__, _kw))
 
         self.typ = 'rt' if typ is None else typ
-        self.Resolver = ruamel.yaml.resolver.VersionedResolver
+        self.Resolver = ruamel.yaml.resolver.VersionedResolver               # type: Any
         self.allow_unicode = True
         self.Reader = None       # type: Any
         self.Scanner = None      # type: Any
@@ -99,6 +99,7 @@ class YAML(object):
         self.prefix_colon = None
         self.version = None
         self.preserve_quotes = None
+        self.allow_duplicate_keys = False  # duplicate keys in map, set
         self.encoding = None
         self.explicit_start = None
         self.explicit_end = None
@@ -138,7 +139,7 @@ class YAML(object):
                     #     # pathlib.Path() instance
                     #     setattr(self, attr, CParser(self._stream))
                     # else:
-                    setattr(self, attr, CParser(self._stream))  # type: ignore
+                    setattr(self, attr, CParser(self._stream))
                     # self._parser = self._composer = self
                     # print('scanner', self.loader.scanner)
 
@@ -157,8 +158,9 @@ class YAML(object):
         # type: () -> Any
         attr = '_' + sys._getframe().f_code.co_name
         if not hasattr(self, attr):
-            setattr(self, attr, self.Constructor(
-                preserve_quotes=self.preserve_quotes, loader=self))
+            cnst = self.Constructor(preserve_quotes=self.preserve_quotes, loader=self)
+            cnst.allow_duplicate_keys = self.allow_duplicate_keys
+            setattr(self, attr, cnst)
         return getattr(self, attr)
 
     @property
@@ -289,38 +291,41 @@ class YAML(object):
                         CParser.__init__(selfx, stream)
                         selfx._parser = selfx._composer = selfx
                         self.Constructor.__init__(selfx, loader=selfx)
+                        selfx.allow_duplicate_keys = self.allow_duplicate_keys
                         rslvr.__init__(selfx, loadumper=selfx)
                 self._stream = stream
                 loader = XLoader(stream)
                 return loader, loader
         return self.constructor, self.parser
 
-    def dump(self, data, stream=None):
-        # type: (Any, StreamType) -> Any
-        return self.dump_all([data], stream)
+    def dump(self, data, stream, _kw=enforce, transform=None):
+        # type: (Any, StreamType, Any, Any) -> Any
+        return self.dump_all([data], stream, _kw, transform=transform)
 
-    def dump_all(self, documents, stream=None):
-        # type: (Any, StreamType) -> Any
+    def dump_all(self, documents, stream, _kw=enforce, transform=None):
+        # type: (Any, StreamType, Any, Any) -> Any
         """
         Serialize a sequence of Python objects into a YAML stream.
         If stream is None, return the produced string instead.
         """
-        # The stream should have the methods `write` and possibly `flush`.
         if not hasattr(stream, 'write') and hasattr(stream, 'open'):
             # pathlib.Path() instance
             with stream.open('w') as fp:   # type: ignore
-                return self.dump_all(documents, fp)
-        getvalue = None
+                return self.dump_all(documents, fp, _kw, transform=transform)
+        if _kw is not enforce:
+            raise TypeError("{}.dump(_all) takes two positional argument but at least "
+                            "three were given ({!r})".format(self.__class__.__name__, _kw))
+        # The stream should have the methods `write` and possibly `flush`.
         if self.top_level_colon_align is True:
             tlca = max([len(str(x)) for x in documents[0]])  # type: Any
         else:
             tlca = self.top_level_colon_align
-        if stream is None:
+        if transform is not None:
+            fstream = stream
             if self.encoding is None:
                 stream = StringIO()
             else:
                 stream = BytesIO()
-            getvalue = stream.getvalue
         serializer, representer, emitter = \
             self.get_serializer_representer_emitter(stream, tlca)
         try:
@@ -340,8 +345,8 @@ class YAML(object):
                 # self.dumper.dispose()  # cyaml
             delattr(self, "_serializer")
             delattr(self, "_emitter")
-        if getvalue is not None:
-            return getvalue()
+        if transform:
+            fstream.write(transform(stream.getvalue()))
         return None
 
     def get_serializer_representer_emitter(self, stream, tlca):
@@ -472,9 +477,9 @@ def load(stream, Loader=None, version=None, preserve_quotes=None):
         Loader = UnsafeLoader
     loader = Loader(stream, version, preserve_quotes=preserve_quotes)
     try:
-        return loader._constructor.get_single_data()
+        return loader._constructor.get_single_data()  # type: ignore
     finally:
-        loader._parser.dispose()
+        loader._parser.dispose()  # type: ignore
 
 
 def load_all(stream, Loader=None, version=None, preserve_quotes=None):
@@ -488,10 +493,10 @@ def load_all(stream, Loader=None, version=None, preserve_quotes=None):
         Loader = UnsafeLoader
     loader = Loader(stream, version, preserve_quotes=preserve_quotes)
     try:
-        while loader._constructor.check_data():
-            yield loader._constructor.get_data()
+        while loader._constructor.check_data():  # type: ignore
+            yield loader._constructor.get_data()  # type: ignore
     finally:
-        loader._parser.dispose()
+        loader._parser.dispose()  # type: ignore
 
 
 def safe_load(stream, version=None):
